////////////////////////////////////////////////////////////////////////////////
// Module    : WinX20.c
// Function  : DLL Startup/Shutdown functions and pointers to DLL functions.
// Usage     : Linked by all Windows apps that access SX20.DLL functions.
// Author    : Jim Lamberson
// Copyright : (C) 2002 Sensoray
////////////////////////////////////////////////////////////////////////////////

/********************* IMPORTANT NOTE FOR MFC PROJECTS *********************
If you are developing an MFC project, you must include the option
"Not using precompiled headers" in the project properties for this file.
***************************************************************************/

// Enable WinX20.h function pointers to be created instead of referenced as externals.
#define XFUNCX20( RTNTYPE, FUNCNAME )   RTNTYPE ( __stdcall *FUNCNAME )

#include <windows.h>

#define USING_LOADLIBRARY_METHOD
#include "sx20func.h"

static HINSTANCE hlib;

///////////////////////////////////////////////////////////////////
// Initialize all pointers to exported DLL functions.
// Returns 0 if pointers successfully initialized.

#define GETFPTR(FPTR,FNAM)  if ( ( (FARPROC)FPTR = GetProcAddress(hlib, FNAM) ) == NULL ) return -1;

DWORD GetFuncPtrs(void)
{
#ifdef USING_LOADLIBRARY_METHOD
	
	// Status and control functions.
  GETFPTR( SX20_AddressGet, "SX20_AddressGet" );
  GETFPTR( SX20_ErrFlagsGet,  "SX20_ErrFlagsGet"  );
  GETFPTR( SX20_BoardOpen,  "SX20_BoardOpen"  );
  GETFPTR( SX20_BoardClose, "SX20_BoardClose" );
  GETFPTR( SX20_BoardIenGet,  "SX20_BoardIenGet"  );
  GETFPTR( SX20_BoardIenSet,  "SX20_BoardIenSet"  );
  GETFPTR( SX20_BoardReset, "SX20_BoardReset" );

  // Direct register access functions.
  GETFPTR( SX20_PortDwordGet, "SX20_PortDwordGet" );
  GETFPTR( SX20_PortDwordSet, "SX20_PortDwordSet" );

  // Digital I/O functions.
  GETFPTR( SX20_ChanDioGet, "SX20_ChanDioGet" );
  GETFPTR( SX20_ChanDioSet, "SX20_ChanDioSet" );
  GETFPTR( SX20_ChanEdgeGet,  "SX20_ChanEdgeGet"  );
  GETFPTR( SX20_ChanEdgeSet,  "SX20_ChanEdgeSet"  );
  GETFPTR( SX20_ChanArmGet, "SX20_ChanArmGet" );
  GETFPTR( SX20_ChanArmSet, "SX20_ChanArmSet" );
  GETFPTR( SX20_ChanCapGet, "SX20_ChanCapGet" );
  GETFPTR( SX20_ChanIenGet, "SX20_ChanIenGet" );
  GETFPTR( SX20_ChanIenSet, "SX20_ChanIenSet" );
  GETFPTR( SX20_GroupDioGet,  "SX20_GroupDioGet"  );
  GETFPTR( SX20_GroupDioSet,  "SX20_GroupDioSet"  );
  GETFPTR( SX20_OctetDdrGet,  "SX20_OctetDdrGet"  );
  GETFPTR( SX20_OctetDdrSet,  "SX20_OctetDdrSet"  );
  GETFPTR( SX20_LoeGet,   "SX20_LoeGet"   );
  GETFPTR( SX20_LoeSet,   "SX20_LoeSet"   );
#endif
  return 0;   // Indicate all is OK.
}

////////////////////////////////////////////////////////////////////
// Open SX20.DLL and get pointers to exported DLL functions.

DWORD SX20_DLLOpen()
{
#ifdef USING_LOADLIBRARY_METHOD
  // Dynamically link to SX20.DLL, exit with error if link failed.
  hlib = LoadLibrary( "SX20.DLL" );
  if ( hlib == NULL )
    return ERR_LOAD_DLL;

  // Fill pointers to SX20.DLL functions, exit with error if attempt failed.
  if ( GetFuncPtrs() )
  {
    FreeLibrary( hlib );
    return ERR_FUNCADDR;
  }
#endif
  // Normal return.
  return 0;
}

///////////////////////////////////////////////////////////////////
// Release dynamic link to SX20.DLL.

VOID SX20_DLLClose()
{
#ifdef USING_LOADLIBRARY_METHOD
  // Unlink from SX20.DLL.
  if ( hlib )
  {

    // Disconnect.
    FreeLibrary( hlib );
  }
#endif
}
